/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * Copyright (C) 2010 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Jan Görig <jgorig@redhat.com>
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <pthread.h>
#include <signal.h>
#include <unistd.h>
#include "libnmserver.h"
#include "libnmserver-internal.h"
#include "libnmserver-dbus.h"

nmserver_result_t nmserver_control_create(nmserver_control_t **pcontrol,
					  void *data)
{
	nmserver_control_t *control;
	nmserver_result_t res;

	assert(pcontrol != NULL && *pcontrol == NULL);

	control = malloc(sizeof(*control));
	if (control == NULL)
		return NMSERVER_R_MEMORY;

	nslist_init(&control->nameservers);
	control->nscallback = NULL;
	control->errcallback = NULL;
	control->is_threaded = false;

	control->conn = nmserver_dbus_init();
	if (control->conn == NULL) {
		res = NMSERVER_R_CONN;
		goto cleanup;
	}

	if (pipe(control->comm_pipe) == -1) {
		res = NMSERVER_R_PIPE;
		goto cleanup;
	}

	control->data = data;

	*pcontrol = control;

	return NMSERVER_R_OK;

cleanup:
	if (control->conn != NULL)
		nmserver_dbus_close(control->conn);
	free(control);

	return res;
}

void nmserver_control_destroy(nmserver_control_t **pcontrol)
{
	nmserver_control_t *control;

	assert(pcontrol != NULL && *pcontrol != NULL);

	control = *pcontrol;

	if (write(control->comm_pipe[1], "Q", 1) != 1) {
		fprintf(stderr, "libnmserver: can't write to pipe!\n");
		return;
	}

	close(control->comm_pipe[1]);

	if (control->is_threaded) {
		pthread_join(control->thread, NULL); /* XXX Check retcode? */
		control->is_threaded = false;
	}

	nmserver_dbus_close(control->conn);
	nslist_free(&control->nameservers);

	free(control);
	*pcontrol = NULL;
}

/* Register callback for nameservers */
void nmserver_nscallback_register(nmserver_control_t *control,
				  nmserver_nscallback_t callback)
{
	assert(control != NULL);

	control->nscallback = callback;
}

/* Register error callback */
void nmserver_errcallback_register(nmserver_control_t *control,
				   nmserver_errcallback_t callback)
{
	assert(control != NULL);

	control->errcallback = callback;
}

/* Runs library loop in new thread */
nmserver_result_t nmserver_threaded_run(nmserver_control_t *control)
{
	int ret;

	assert(control != NULL);

	ret = pthread_create(&control->thread, NULL,
			     (void* (*)(void*)) nmserver_run, control);
	if (ret != 0)
		return NMSERVER_R_FAILURE; /* XXX Convert to nmserver_result */

	control->is_threaded = true;

	return NMSERVER_R_OK;
}

/* Runs library loop  */
void nmserver_run(nmserver_control_t *control)
{
	int ret;

	assert(control != NULL);
#if 0	/* This assert is too strong */
	assert(control->callback_ipv4_ns != NULL);
#endif

	ret = nmserver_dbus_callback_nameservers(control);
	if (ret != NMSERVER_R_OK && control->errcallback != NULL)
		control->errcallback(ret);

	nmserver_dbus_loop(control);
}
